@@ -8,15 +8,25 @@
 #
 # - one single function for mean + cov
 # - handle 'fixed.x' exogenous covariates
+
+
 # - YR 3 Dec 2015: allow for conditional.x = TRUE
 
-# generic public function
+# generic public function (not exported yet)
 # input for lavGamma can be lavobject, lavdata, data.frame, or matrix
 lavGamma <- function(object, group = NULL, missing = "listwise",
                      ov.names.x = NULL, fixed.x = FALSE, conditional.x = FALSE,
                      meanstructure = FALSE, slopestructure = FALSE,
+                     gamma.n.minus.one = FALSE, gamma.unbiased = FALSE,
+                     ADF = TRUE, NT.rescale = FALSE,
                      Mplus.WLS = FALSE, add.labels) {
 
+    # no support for missing = "ml" (and model-based Gamma) yet
+    if(missing != "listwise") {
+        ### FIXME!!!!!
+        return(NULL) # for now!
+    }
+
     if(inherits(object, "lavaan")) {
         lavdata <- object@Data
         if(missing(missing)) {
@@ -25,8 +35,21 @@ lavGamma <- function(object, group = NULL, missing = "listwise",
                 ### FIXME!!!!!
                 return(NULL) # for now!
             }
-        } else {
-            missing <- "listwise"
+        }
+        if(missing(fixed.x)) {
+            fixed.x <- object@Options$fixed.x
+        }
+        if(missing(conditional.x)) {
+            conditional.x <- object@Options$conditional.x
+        }
+        if(missing(meanstructure)) {
+            meanstructure <- object@Options$meanstructure
+        }
+        if(missing(gamma.n.minus.one)) {
+            gamma.n.minus.one <- object@Options$gamma.n.minus.one
+        }
+        if(missing(gamma.unbiased)) {
+            gamma.unbiased <- object@Options$gamma.unbiased
         }
     } else if(inherits(object, "lavData")) {
         lavdata <- object
@@ -54,14 +77,36 @@ lavGamma <- function(object, group = NULL, missing = "listwise",
                     function(g) match(lavdata@ov.names.x[[g]],
                                       lavdata@ov.names[[g]]) )
 
-    OUT <- lapply(seq_len(lavdata@ngroups),
-              function(g) lav_samplestats_Gamma(Y              = Y[[g]],
-                                                x.idx          = x.idx[[g]],
-                                                fixed.x        = fixed.x,
-                                                conditional.x  = conditional.x,
-                                                meanstructure  = meanstructure,
-                                                slopestructure = slopestructure,
-                                                Mplus.WLS      = Mplus.WLS))
+    OUT <- lapply(seq_len(lavdata@ngroups), function(g) {
+        if(length(lavdata@cluster) > 0L) {
+            cluster.idx <- lavdata@Lp[[g]]$cluster.idx[[2]]
+        } else {
+            cluster.idx <- NULL
+        }
+        if(ADF) {
+            out <- lav_samplestats_Gamma(Y              = Y[[g]],
+                                     x.idx          = x.idx[[g]],
+                                     cluster.idx    = cluster.idx,
+                                     fixed.x        = fixed.x,
+                                     conditional.x  = conditional.x,
+                                     meanstructure  = meanstructure,
+                                     slopestructure = conditional.x,
+                                     gamma.n.minus.one = gamma.n.minus.one,
+                                     unbiased       = gamma.unbiased,
+                                     Mplus.WLS      = Mplus.WLS)
+        } else {
+            # FIXME: allow for moments only
+            # and provide COV/MEAN/...
+            out <- lav_samplestats_Gamma_NT(Y       = Y[[g]],
+                                     wt             = NULL, # for now
+                                     rescale        = NT.rescale,
+                                     x.idx          = x.idx[[g]],
+                                     fixed.x        = fixed.x,
+                                     conditional.x  = conditional.x,
+                                     meanstructure  = meanstructure,
+                                     slopestructure = conditional.x)
+        }
+        out})
 
     OUT
 }
@@ -114,7 +159,7 @@ lav_samplestats_Gamma_NT <- function(Y              = NULL, # should include
        (meanstructure || slopestructure)) {
        stopifnot(!is.null(Y))
        if(is.null(wt)) {
-           MEAN <- colMeans(Y)
+           MEAN <- colMeans(Y, na.rm = TRUE)
        } else {
            MEAN <- out$center
        }
@@ -217,6 +262,7 @@ lav_samplestats_Gamma_NT <- function(Y              = NULL, # should include
 #  - new in 0.6-1: if Mu/Sigma is provided, compute 'model-based' Gamma
 #                  (only if conditional.x = FALSE, for now)
 #  - new in 0.6-2: if cluster.idx is not NULL, correct for clustering
+#  - new in 0.6-13: add unbiased = TRUE (for the 'plain' setting only)
 
 # ADF THEORY
 lav_samplestats_Gamma <- function(Y,
@@ -229,10 +275,23 @@ lav_samplestats_Gamma <- function(Y,
                                   meanstructure      = FALSE,
                                   slopestructure     = FALSE,
                                   gamma.n.minus.one  = FALSE,
+                                  unbiased           = FALSE,
                                   Mplus.WLS          = FALSE) {
     # coerce to matrix
     Y <- unname(as.matrix(Y)); N <- nrow(Y); p <- ncol(Y)
 
+    # unbiased?
+    if(unbiased) {
+        if(conditional.x || fixed.x || !is.null(Sigma) ||
+           !is.null(cluster.idx)) {
+            stop("lavaan ERROR: unbiased Gamma only available for the simple (not conditional.x or fixed.x or model-based or clustered) setting.")
+        } else {
+            COV <- COV.unbiased <- cov(Y)
+            COV <- COV * (N-1)/N
+            cov.vech <- lav_matrix_vech(COV)
+        }
+    }
+
     # model-based?
     if(!is.null(Sigma)) {
         stopifnot(!conditional.x)
@@ -266,7 +325,7 @@ lav_samplestats_Gamma <- function(Y,
         if(model.based) {
             Yc <- t( t(Y) - as.numeric(Mu) )
         } else {
-            Yc <- t( t(Y) - colMeans(Y) )
+            Yc <- t( t(Y) - colMeans(Y, na.rm = TRUE) )
         }
 
         # create Z where the rows_i contain the following elements:
@@ -291,7 +350,7 @@ lav_samplestats_Gamma <- function(Y,
             }
             Zc <- t( t(Z) - sigma )
         } else {
-            Zc <- t( t(Z) - colMeans(Z) )
+            Zc <- t( t(Z) - colMeans(Z, na.rm = TRUE) )
         }
 
         # clustered?
@@ -307,7 +366,7 @@ lav_samplestats_Gamma <- function(Y,
     } else if(!conditional.x && fixed.x) {
 
         if(model.based) {
-            Y.bar <- colMeans(Y)
+            Y.bar <- colMeans(Y, na.rm = TRUE)
             res.cov <- ( Sigma[-x.idx, -x.idx, drop = FALSE] -
                          Sigma[-x.idx, x.idx, drop = FALSE] %*%
                           solve(Sigma[x.idx, x.idx, drop = FALSE]) %*%
@@ -315,7 +374,8 @@ lav_samplestats_Gamma <- function(Y,
             res.slopes <- ( solve(Sigma[x.idx, x.idx, drop = FALSE]) %*%
                             Sigma[x.idx, -x.idx, drop = FALSE] )
             res.int <- ( Y.bar[-x.idx] -
-                 as.numeric(colMeans(Y[,x.idx,drop = FALSE]) %*% res.slopes) )
+                         as.numeric(colMeans(Y[,x.idx,drop = FALSE],
+                                    na.rm = TRUE) %*% res.slopes) )
             x.bar <- Y.bar[x.idx]
             yhat.bar <- as.numeric(res.int + as.numeric(x.bar) %*% res.slopes)
             YHAT.bar <- numeric(p)
@@ -348,8 +408,8 @@ lav_samplestats_Gamma <- function(Y,
             yhat <- qr.fitted(QR, Y[, -x.idx, drop = FALSE])
             YHAT <- cbind(yhat, Y[,x.idx])
 
-            Yc <- t( t(Y) - colMeans(Y) )
-            YHATc <- t( t(YHAT) - colMeans(YHAT) )
+            Yc <- t( t(Y) - colMeans(Y, na.rm = TRUE) )
+            YHATc <- t( t(YHAT) - colMeans(YHAT, na.rm = TRUE) )
             idx1 <- lav_matrix_vech_col_idx(p)
             idx2 <- lav_matrix_vech_row_idx(p)
             if(meanstructure) {
@@ -363,7 +423,7 @@ lav_samplestats_Gamma <- function(Y,
                        YHATc[,idx1, drop = FALSE] *
                        YHATc[,idx2, drop = FALSE] )
             }
-            Zc <- t( t(Z) - colMeans(Z) )
+            Zc <- t( t(Z) - colMeans(Z, na.rm = TRUE) )
         }
 
         # clustered?
@@ -438,7 +498,7 @@ lav_samplestats_Gamma <- function(Y,
         if(model.based) {
             Zc <- t( t(Z) - sigma )
         } else {
-            Zc <- t( t(Z) - colMeans(Z) )
+            Zc <- t( t(Z) - colMeans(Z, na.rm = TRUE) )
         }
 
         # clustered?
@@ -483,6 +543,34 @@ lav_samplestats_Gamma <- function(Y,
         Gamma <- Gamma * nC / (nC - 1)
     }
 
+    # unbiased?
+    if(unbiased) {
+        # normal-theory Gamma (cov only)
+        GammaNT.cov <- 2 * lav_matrix_duplication_ginv_pre_post(COV %x% COV)
+
+        if(meanstructure) {
+            Gamma.cov      <- Gamma[-(1:p), -(1:p), drop = FALSE]
+            Gamma.mean.cov <- Gamma[1:p,    -(1:p), drop = FALSE]
+        } else {
+            Gamma.cov <- Gamma
+        }
+
+        # Browne's unbiased DF estimator (COV part)
+        Gamma.u <- ( N*(N-1)/(N-2)/(N-3) * Gamma.cov -
+                     N/(N-2)/(N-3) * ( GammaNT.cov -
+                                       2/(N-1) * tcrossprod(cov.vech) )  )
+        if(meanstructure) {
+            Gamma <- lav_matrix_bdiag(COV,  Gamma.u)
+
+            # 3-rd order:
+            Gamma[1:p,(p+1):ncol(Gamma)] <- Gamma.mean.cov * N/(N-2)
+            Gamma[(p+1):ncol(Gamma),1:p] <- t( Gamma.mean.cov * N/(N-2) )
+
+        } else {
+            Gamma <- Gamma.u
+        }
+    } # unbiased
+
     Gamma
 }
 
