@@ -54,17 +54,69 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     start.time0 <- start.time <- proc.time()[3]; timing <- list()
 
     # 0a. store call
-    mc  <- match.call(expand.dots = TRUE)
+    SYSCALL <- sys.call() # to get main arguments without partial matching
+    mc  <- match.call(expand.dots = TRUE) # sys.call to avoid partial matching
 
     # handle dotdotdot
     dotdotdot <- list(...)
 
+    # catch partial matching of 'cl' (expanded to cluster)
+    if(!is.null(SYSCALL[["cl"]]) && is.null(SYSCALL[["cluster"]]) &&
+       !is.null(mc[["cluster"]])) {
+        mc[["cl"]] <- mc[["cluster"]]
+        mc[["cluster"]] <- cluster <- NULL
+        dotdotdot$cl <- SYSCALL[["cl"]]
+    }
+
     # check data
     if(!is.null(data)) {
         if(inherits(data, "data.frame")) {
             # just in case it is not a traditional data.frame
             data <- as.data.frame(data)
+
+        } else if (inherits(data, "lavMoments")) {
+          # This object must contain summary statistics
+          # e.g., created by lavaan.mi::poolSat
+
+          # set required-data arguments
+          if ("sample.cov" %in% names(data)) {
+            sample.cov  <- data$sample.cov
+          } else stop("When data= is of class lavMoments, it must contain sample.cov")
+
+          if ("sample.nobs" %in% names(data)) {
+            sample.nobs <- data$sample.nobs
+          } else stop("When data= is of class lavMoments, it must contain sample.nobs")
+
+          # check for optional-data arguments
+          if ("sample.mean" %in% names(data)) sample.mean <- data$sample.mean
+          if ("sample.th"   %in% names(data)) sample.th   <- data$sample.th
+          if ("NACOV"       %in% names(data)) NACOV       <- data$NACOV
+          if ("WLS.V"       %in% names(data)) WLS.V       <- data$WLS.V
+
+          # set other args not included in dotdotdot
+          if (length(data$lavOptions)) {
+            newdots <- setdiff(names(data$lavOptions), names(dotdotdot))
+            if (length(newdots)) {
+              for (dd in newdots) dotdotdot[[dd]] <- data$lavOptions[[dd]]
+            }
+          }
+
+          #FIXME: Should WLS.V be an I(dentity) matrix when ULS is requested?
+          #       Unused for point estimates, but still used to scale/shift test
+          # if (!is.null(dotdotdot$estimator)) {
+          #   if (grepl(pattern = "ULS", x = toupper(dotdotdot$estimator[1L]) ) &&
+          #       !is.null(WLS.V)) {
+          #     ## set to diagonal
+          #     if (is.list(WLS.V)) {
+          #       WLS.V <- lapply(WLS.V, function(w) {diag(w) <- 1 ; return(w) } )
+          #     } else diag(WLS.V) <- 1
+          #   }
+          # }
+
+          # get rid of data= argument
+          data <- NULL
         }
+
         if(is.function(data)) {
             stop("lavaan ERROR: data is a function; it should be a data.frame")
         }
@@ -287,6 +339,17 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         }
     }
 
+    # sanity check: we do not support latent interaction yet (using the :)
+    lv.int.idx <- which(grepl(":", LV.names))
+    if(length(lv.int.idx) > 0L) {
+        txt <- c("Interaction terms involving latent variables (",
+                 LV.names[lv.int.idx[1]], ") are not supported.",
+                 " You may consider creating product indicators to define ",
+                 "the latent interaction term. See the indProd() function ",
+                 "in the semTools package.")
+        stop(lav_txt2message(txt, header = "lavaan ERROR:"))
+    }
+
     # handle ov.names.l
     if(any(FLAT$op == ":" & tolower(FLAT$lhs) == "level")) {
 
@@ -408,6 +471,20 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     if(!is.null(slotOptions)) {
         lavoptions <- slotOptions
 
+        # backwards compatibility
+        if(!is.null(lavoptions$categorical)) {
+            lavoptions$.categorical <- lavoptions$categorical
+            lavoptions$categorical <- NULL
+        }
+        if(!is.null(lavoptions$clustered)) {
+            lavoptions$.clustered <- lavoptions$clustered
+            lavoptions$clustered <- NULL
+        }
+        if(!is.null(lavoptions$multilevel)) {
+            lavoptions$.multilevel <- lavoptions$multilevel
+            lavoptions$multilevel <- NULL
+        }
+
         # but what if other 'options' are given anyway (eg 'start = ')?
         # give a warning!
         if(length(dotdotdot) > 0L) {
@@ -439,47 +516,62 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         opt <- modifyList(opt, dotdotdot)
 
         # no data?
-        if(is.null(data) && is.null(sample.cov)) {
+        if(is.null(slotData) && is.null(data) && is.null(sample.cov)) {
             opt$bounds <- FALSE
         }
 
+        # only sample moments?
+        if(!is.null(slotData) && !slotData@data.type == "full") {
+            opt$missing <- "listwise"
+        } else if(is.null(slotData) && is.null(data)) {
+            opt$missing <- "listwise"
+        }
+
         # categorical mode?
-        opt$categorical <- FALSE
+        opt$.categorical <- FALSE
         if(any(FLAT$op == "|")) {
-            opt$categorical <- TRUE
+            opt$.categorical <- TRUE
         } else if(!is.null(data) && length(ordered) > 0L) {
-            opt$categorical <- TRUE
+            opt$.categorical <- TRUE
         } else if(!is.null(sample.th)) {
-            opt$categorical <- TRUE
+            opt$.categorical <- TRUE
         } else if(is.data.frame(data)) {
             # first check if we can find ov.names.y in Data
             OV.names.y <- unique(unlist(ov.names.y))
+            # remove possible interaction terms involving an y term
+            int.idx <- which(grepl(":", OV.names.y))
+            if(length(int.idx) > 0L) {
+                OV.names.y <- OV.names.y[-int.idx]
+            }
             idx.missing <- which(!(OV.names.y %in% names(data)))
             if(length(idx.missing)) {
                 stop("lavaan ERROR: missing observed variables in dataset: ",
                     paste(OV.names.y[idx.missing], collapse=" "))
             }
             if(any(sapply(data[, OV.names.y], inherits, "ordered")) ) {
-                opt$categorical <- TRUE
+                opt$.categorical <- TRUE
             }
         }
+        if(tolower(opt$estimator) == "catml") {
+            opt$.categorical <- FALSE
+        }
 
         # clustered?
         if(length(cluster) > 0L) {
-            opt$clustered <- TRUE
+            opt$.clustered <- TRUE
         } else {
-            opt$clustered <- FALSE
+            opt$.clustered <- FALSE
         }
 
         # multilevel?
         if(length(ov.names.l) > 0L && length(ov.names.l[[1]]) > 1L) {
-            opt$multilevel <- TRUE
+            opt$.multilevel <- TRUE
         } else {
-            opt$multilevel <- FALSE
+            opt$.multilevel <- FALSE
         }
 
         # sampling weights? force MLR
-        if(!is.null(sampling.weights) && !opt$categorical &&
+        if(!is.null(sampling.weights) && !opt$.categorical &&
            opt$estimator %in% c("default", "ML")) {
             opt$estimator <- "MLR"
         }
@@ -611,6 +703,12 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         }
         # more needed?
     }
+    # sanity check
+    if(!is.null(slotParTable) || inherits(model, "lavaan")) {
+        if(ngroups != lavdata@ngroups) {
+            stop("lavaan ERROR: mismatch between number of groups in data, and number of groups in model.")
+        }
+    }
     timing$Data <- (proc.time()[3] - start.time)
     start.time <- proc.time()[3]
     if(lavoptions$verbose) {
@@ -659,10 +757,18 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         #    tmp <- vnames(FLAT, type = "ov.x", ov.x.fatal = TRUE)
         #}
 
+        if(lavoptions$estimator == "catML") {
+            lavoptions$meanstructure <- FALSE
+            DataOV <- lavdata@ov
+            DataOV$type <- rep("numeric", length(DataOV$type))
+        } else {
+            DataOV <- lavdata@ov
+        }
+
         lavpartable <-
             lavaanify(model            = FLAT,
                       constraints      = constraints,
-                      varTable         = lavdata@ov,
+                      varTable         = DataOV,
                       ngroups          = lavdata@ngroups,
 
                       meanstructure    = lavoptions$meanstructure,
@@ -675,6 +781,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                       conditional.x    = lavoptions$conditional.x,
                       fixed.x          = lavoptions$fixed.x,
                       std.lv           = lavoptions$std.lv,
+                      correlation      = lavoptions$correlation,
                       effect.coding    = lavoptions$effect.coding,
                       ceq.simple       = lavoptions$ceq.simple,
                       parameterization = lavoptions$parameterization,
@@ -718,7 +825,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     # was used, but the user has forgotten some variances/intercepts...
     if(is.null(slotParTable)) {
         junk <- lav_partable_check(lavpartable,
-                                   categorical = lavoptions$categorical,
+                                   categorical = lavoptions$.categorical,
                                    warn = TRUE)
     }
 
@@ -785,14 +892,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                            sample.nobs   = sample.nobs,
                            ov.names      = ov.names,
                            ov.names.x    = ov.names.x,
-                           estimator     = lavoptions$estimator,
-                           mimic         = lavoptions$mimic,
-                           meanstructure = lavoptions$meanstructure,
-                           group.w.free  = lavoptions$group.w.free,
                            WLS.V         = WLS.V,
                            NACOV         = NACOV,
-                           ridge         = lavoptions$ridge,
-                           rescale       = lavoptions$sample.cov.rescale)
+                           lavoptions    = lavoptions)
         if(lavoptions$verbose) {
             cat(" done.\n")
         }
@@ -830,6 +932,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                                            lavsamplestats = lavsamplestats,
                                            lavpta         = lavpta,
                                            lavoptions     = lavoptions)
+                if(lavoptions$debug) {
+                    print(out)
+                }
                 h1.implied      <- out$implied
                 h1.loglik       <- out$logl$loglik
                 h1.loglik.group <- out$logl$loglik.group
@@ -932,6 +1037,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             START <- lav_start(start.method   = lavoptions$start,
                                lavpartable    = lavpartable,
                                lavsamplestats = lavsamplestats,
+                               lavh1          = lavh1,
                                model.type     = lavoptions$model.type,
                                reflect      = FALSE,
                                #order.lv.by  = lavoptions$rotation.args$order.lv.by,
@@ -1278,8 +1384,17 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             cat("lavoptim           ... start:\n")
         }
 
+        # non-iterative methods (fabin, ...)
+        if(lavoptions$optim.method == "noniter") {
+            x <- try(lav_optim_noniter(lavmodel       = lavmodel,
+                                       lavsamplestats = lavsamplestats,
+                                       lavdata        = lavdata,
+                                       lavpartable    = lavpartable,
+                                       lavpta         = lavpta,
+                                       lavoptions     = lavoptions),
+                     silent = TRUE)
         # EM for multilevel models
-        if(lavoptions$optim.method == "em") {
+        } else if(lavoptions$optim.method == "em") {
             # multilevel only for now
             stopifnot(lavdata@nlevels > 1L)
             x <- try(lav_mvnorm_cluster_em_h0(lavsamplestats = lavsamplestats,
@@ -1296,11 +1411,6 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         # Gauss-Newton
         } else if(lavoptions$optim.method == "gn") {
             # only tested for DLS (for now)
-            #if(lavoptions$estimator != "DLS") {
-            #    stop("lavaan ERROR: optim.method = ", dQuote("gn"),
-            #         " is only available for estimator = ", dQuote("DLS"),
-            #         " (for now).")
-            #}
             x <- try(lav_optim_gn(lavmodel       = lavmodel,
                                   lavsamplestats = lavsamplestats,
                                   lavdata        = lavdata,
@@ -1900,7 +2010,19 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                   external     = list()               # empty list
                  )
 
-
+    # if model.type = "efa", add standardized solution to partable
+    if( (.hasSlot(lavmodel, "nefa")) && (lavmodel@nefa > 0L)) {
+        if(lavoptions$verbose) {
+            cat("computing standardized solution ... ")
+        }
+        STD <- standardizedSolution(lavaan, remove.eq = FALSE,
+                                    remove.ineq = FALSE, remove.def = FALSE)
+        if(lavoptions$verbose) {
+            cat(" done.\n")
+        }
+        lavaan@ParTable$est.std <- STD$est.std
+        lavaan@ParTable$se.std  <- STD$se
+    }
 
     # post-fitting check of parameters
     if(!is.null(lavoptions$check.post) && lavoptions$check.post &&
@@ -1953,8 +2075,16 @@ cfa <- sem <- function(# user-specified model: can be syntax, parameter Table
                        # options (dotdotdot)
                        ...) {
 
+    SYSCALL <- sys.call() # no partial matching
     mc <- match.call(expand.dots = TRUE)
 
+    # catch partial matching of 'cl' (expanded to cluster)
+    if(!is.null(SYSCALL[["cl"]]) && is.null(SYSCALL[["cluster"]]) &&
+       !is.null(mc[["cluster"]])) {
+        mc[["cl"]] <- mc[["cluster"]]
+        mc[["cluster"]] <- NULL
+    }
+
     # set model.type
     mc$model.type      = as.character( mc[[1L]] )
     if(length(mc$model.type) == 3L) {
@@ -2020,8 +2150,16 @@ growth <- function(# user-specified model: can be syntax, parameter Table
                    # options (dotdotdot)
                    ...) {
 
+    SYSCALL <- sys.call() # no partial matching
     mc <- match.call(expand.dots = TRUE)
 
+    # catch partial matching of 'cl' (expanded to cluster)
+    if(!is.null(SYSCALL[["cl"]]) && is.null(SYSCALL[["cluster"]]) &&
+       !is.null(mc[["cluster"]])) {
+        mc[["cl"]] <- mc[["cluster"]]
+        mc[["cluster"]] <- NULL
+    }
+
     # set model.type
     mc$model.type      = as.character( mc[[1L]] )
     if(length(mc$model.type) == 3L) {
